// includes

#include <string.h>

#include "common.h"
#include "bitboard.h"

// variables

bitboard_t bitboards;

/*  Data used for generating MoveArray  */

static const int dir[8][8] =
{
  { 0, 0, 0, 0, 0, 0, 0, 0 },
  { 9, 11, 0, 0, 0, 0, 0, 0 },
  { -21, -19, -12, -8, 8, 12, 19, 21 },
  { -11, -9, 9, 11, 0, 0, 0, 0 },
  { -10, -1, 1, 10, 0, 0, 0, 0 },
  { -11, -10, -9, -1, 1, 9, 10, 11 },
  { -11, -10, -9, -1, 1, 9, 10, 11 },
  { -9, -11, 0, 0, 0, 0, 0, 0 }
};
static const int ndir[8] = 
{ 0, 2, 8, 4, 4, 8, 8, 2 };

static const int map[120] =
{
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1,  0,  1,  2,  3,  4,  5,  6,  7, -1,
  -1,  8,  9, 10, 11, 12, 13, 14, 15, -1,
  -1, 16, 17, 18, 19, 20, 21, 22, 23, -1,
  -1, 24, 25, 26, 27, 28, 29, 30, 31, -1,
  -1, 32, 33, 34, 35, 36, 37, 38, 39, -1,
  -1, 40, 41, 42, 43, 44, 45, 46, 47, -1,
  -1, 48, 49, 50, 51, 52, 53, 54, 55, -1,
  -1, 56, 57, 58, 59, 60, 61, 62, 63, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 
};


// functions

static void InitBitPosArray (void)
{
   BitBoard b;
   int i;

   b = (BitBoard) 1;  
   for (i = 63; i >= 0; i--, b <<= 1)
   {
      bitboards.BitPosArray[i] = b;
      bitboards.NotBitPosArray[i] = ~b;
   }
}
 
static void InitMoveArray (void)
{
   int piece, fsq, tsq, f, t, n;
   BitBoard *b;

   for (piece = pawn; piece <= bpawn; piece++)
   {
      for (fsq = 0; fsq < 120; fsq++)
      {
         if ((f = map[fsq]) == -1) continue;
         b = &bitboards.MoveArray[piece][f];
         *b = NULLBITBOARD;
         for (n = 0; n < ndir[piece]; n++)
         {
            tsq = fsq;
            do
            {
               tsq += dir[piece][n];
               if ((t = map[tsq]) != -1)
                  SETBIT (*b, t);
            } while (range[piece] && t != -1);
         }
      }
   }
}

static void InitRay (void)
{
   int piece, fsq, tsq, f, t, n, ray;
   BitBoard *b;

   memset (directions, -1, sizeof (directions));
   for (fsq = 0; fsq < 120; fsq++)
   {
      if ((f = map[fsq]) == -1) continue;
      ray = -1;
      for (piece = bishop; piece <= rook; piece++)
      {
         for (n = 0; n < ndir[piece]; n++)
         {
            b = &bitboards.Ray[f][++ray];
            *b = NULLBITBOARD;
            tsq = fsq;
            do
            {
               tsq += dir[piece][n];
               if ((t = map[tsq]) != -1)
	       {
                  SETBIT (*b, t);
	          directions[f][t] = ray;
	       }
            } while (t != -1);
         }
      }
   }
}


static void InitFromToRay (void)
{
   int piece, fsq, tsq, f, t, n;
   BitBoard *b;

   memset (bitboards.FromToRay, 0, sizeof (bitboards.FromToRay));
   for (piece = bishop; piece <= rook; piece++)
   {
      for (fsq = 0; fsq < 120; fsq++)
      {
         if ((f = map[fsq]) == -1) continue;
         for (n = 0; n < ndir[piece]; n++)
         {
            tsq = fsq;
            t = map[tsq];
            do
            {
               b = &bitboards.FromToRay[f][t];
               tsq += dir[piece][n];         
               if ((t = map[tsq]) != -1)
               {
                  SETBIT (bitboards.FromToRay[f][t], t);
                  bitboards.FromToRay[f][t] |= *b;
               }
            } while (t != -1);
         }
      }
   }
}


static void InitRankFileBit (void)
{
   BitBoard b;
   int i;

   i = 8;
   b = (BitBoard) 255;
   while (i--)
   {
      bitboards.RankBit[i] = b;
      b <<= 8;
   }
   
   i = 8;
   b = ULL(0x0101010101010101);
   while (i--)
   {
      bitboards.FileBit[i] = b;
      b <<= 1;
   }
}

static void InitRandomMasks (void)
{
  bitboards.mask_kr_trapped_w[0]=bitboards.BitPosArray[H2];
  bitboards.mask_kr_trapped_w[1]=bitboards.BitPosArray[H1]|bitboards.BitPosArray[H2];
  bitboards.mask_kr_trapped_w[2]=bitboards.BitPosArray[G1]|bitboards.BitPosArray[H1]|bitboards.BitPosArray[H2];
  bitboards.mask_qr_trapped_w[0]=bitboards.BitPosArray[A2];
  bitboards.mask_qr_trapped_w[1]=bitboards.BitPosArray[A1]|bitboards.BitPosArray[A2];
  bitboards.mask_qr_trapped_w[2]=bitboards.BitPosArray[A1]|bitboards.BitPosArray[B1]|bitboards.BitPosArray[A2];
  bitboards.mask_kr_trapped_b[0]=bitboards.BitPosArray[H7];
  bitboards.mask_kr_trapped_b[1]=bitboards.BitPosArray[H8]|bitboards.BitPosArray[H7];
  bitboards.mask_kr_trapped_b[2]=bitboards.BitPosArray[H8]|bitboards.BitPosArray[G8]|bitboards.BitPosArray[H7];
  bitboards.mask_qr_trapped_b[0]=bitboards.BitPosArray[A7];
  bitboards.mask_qr_trapped_b[1]=bitboards.BitPosArray[A8]|bitboards.BitPosArray[A7];
  bitboards.mask_qr_trapped_b[2]=bitboards.BitPosArray[A8]|bitboards.BitPosArray[B8]|bitboards.BitPosArray[A7];


   SETBIT (bitboards.stonewall[white], D4); /* SMC */
   SETBIT (bitboards.stonewall[white], E3); /* SMC */
   SETBIT (bitboards.stonewall[white], F4); /* SMC */

   SETBIT (bitboards.stonewall[black], D5); /* SMC */
   SETBIT (bitboards.stonewall[black], E6); /* SMC */
   SETBIT (bitboards.stonewall[black], F5); /* SMC */

   bitboards.rings[0] = ULL(0x0000001818000000); /* 4 & 5 th file; 4 & 5 th rank */
   bitboards.rings[1] = ULL(0x00003C24243C0000); /* 3 & 6 th file; 3 & 6 th rank */
   bitboards.rings[2] = ULL(0x007E424242427E00); /* 2 & 7 th file; 2 & 7 th rank */
   bitboards.rings[3] = ULL(0xFF818181818181FF); /* 1 & 8 th file; 1 & 8 th rank */

   bitboards.boxes[0] = ULL(0x00003C3C3C3C0000); /* rings[0] | rings[1] */
   bitboards.boxes[1] = ULL(0x007E7E7E7E7E7E00); /* rings[0] | rings[1] | rings[2] */

   bitboards.boardhalf[white] = bitboards.RankBit[0]|bitboards.RankBit[1]|bitboards.RankBit[2]|bitboards.RankBit[3];
   bitboards.boardhalf[black] = bitboards.RankBit[4]|bitboards.RankBit[5]|bitboards.RankBit[6]|bitboards.RankBit[7];
   bitboards.boardside[ks] = bitboards.FileBit[4]|bitboards.FileBit[5]|bitboards.FileBit[6]|bitboards.FileBit[7];
   bitboards.boardside[qs] = bitboards.FileBit[0]|bitboards.FileBit[1]|bitboards.FileBit[2]|bitboards.FileBit[3];

   bitboards.WhiteSquares= ULL(0x55AA55AA55AA55AA);
   bitboards.BlackSquares= ULL(0xAA55AA55AA55AA55);

}

static void InitPassedPawnMask (void)
{
   unsigned int sq;

   memset (bitboards.PassedPawnMask, 0, sizeof (bitboards.PassedPawnMask));

   /*  Do for white pawns first */
   for (sq = 0; sq < 64; sq++)
   {
      bitboards.PassedPawnMask[white][sq] = bitboards.Ray[sq][7];
      if (ROW(sq) != 0)
	bitboards.PassedPawnMask[white][sq] |= bitboards.Ray[sq-1][7];
      if (ROW(sq) != 7)
         bitboards.PassedPawnMask[white][sq] |= bitboards.Ray[sq+1][7];
   }

   /*  Do for black pawns */
   for (sq = 0; sq < 64; sq++)
   {
     bitboards.PassedPawnMask[black][sq] = bitboards.Ray[sq][4];
      if (ROW(sq) != 0)
         bitboards.PassedPawnMask[black][sq] |= bitboards.Ray[sq-1][4];
      if (ROW(sq) != 7)
         bitboards.PassedPawnMask[black][sq] |= bitboards.Ray[sq+1][4];
   }
}

static void InitIsolaniMask (void)

/**************************************************************************
 *
 *  IMPORTANT:!!!!
 *  Make sure this routine is called AFTER InitRankFileBit().
 *
 **************************************************************************/
{
   int i;

   bitboards.IsolaniMask[0] = bitboards.FileBit[1];
   bitboards.IsolaniMask[7] = bitboards.FileBit[6];
   for (i = 1; i <= 6; i++)
      bitboards.IsolaniMask[i] = bitboards.FileBit[i-1] | bitboards.FileBit[i+1];
      
}

static void InitSquarePawnMask (void)
{
   unsigned int sq;
   int len, i, j;

   memset (bitboards.SquarePawnMask, 0, sizeof (bitboards.PassedPawnMask));
   for (sq = 0; sq < 64; sq++)
   {
      /*  White mask  */
      len = 7 - RANK (sq);
      i = MAX (sq & 56, sq - len);
      j = MIN (sq | 7, sq + len);
      while (i <= j)
      {
         bitboards.SquarePawnMask[white][sq] |= (bitboards.BitPosArray[i] | bitboards.FromToRay[i][i|56]);
         i++;
      }

      /*  Black mask  */
      len = RANK (sq);
      i = MAX (sq & 56, sq - len);
      j = MIN (sq | 7, sq + len);
      while (i <= j)
      {
         bitboards.SquarePawnMask[black][sq] |= (bitboards.BitPosArray[i] | bitboards.FromToRay[i][i&7]);
         i++;
      }
   }

   /*  For pawns on 2nd rank, they have same mask as pawns on 3rd rank */
   for (sq = A2; sq <= H2; sq++)
      bitboards.SquarePawnMask[white][sq] = bitboards.SquarePawnMask[white][sq+8];
   for (sq = A7; sq <= H7; sq++)
      bitboards.SquarePawnMask[black][sq] = bitboards.SquarePawnMask[black][sq-8];
}

 

static void InitRotAtak (void)
/**************************************************************************
 *
 *  The attack tables for a normal chessboard and the rotated chess board
 *  are calculated here.
 *
 **************************************************************************/
{
   int sq, map, sq1, sq2;
   int cmap[8] = { 128, 64, 32, 16, 8, 4, 2, 1 };
   int rot1[8] = { A1, A2, A3, A4, A5, A6, A7, A8 };
   int rot2[8] = { A1, B2, C3, D4, E5, F6, G7, H8 };
   int rot3[8] = { A8, B7, C6, D5, E4, F3, G2, H1 };
   
   for (sq = A1; sq <= H1; sq++)
   {
      for (map = 0; map < 256; map++)
      {
	 bitboards.Rook00Atak[sq][map] = 0;
	 bitboards.Rook90Atak[rot1[sq]][map] = 0;
	 bitboards.Bishop45Atak[rot2[sq]][map] = 0;
	 bitboards.Bishop315Atak[rot3[sq]][map] = 0;
	    sq1 = sq2 = sq;
	    while (sq1 > 0)
	    {
	       if (cmap[--sq1] & map)
	          break;
	    }
	    while (sq2 < 7)
	    {
	       if (cmap[++sq2] & map)
	          break;
	    }
	    bitboards.Rook00Atak[sq][map] = 
		bitboards.FromToRay[sq][sq1] | bitboards.FromToRay[sq][sq2];
	    bitboards.Rook90Atak[rot1[sq]][map] =
		bitboards.FromToRay[rot1[sq]][rot1[sq1]] | 
		bitboards.FromToRay[rot1[sq]][rot1[sq2]];
	    bitboards.Bishop45Atak[rot2[sq]][map] =
		bitboards.FromToRay[rot2[sq]][rot2[sq1]] |
		bitboards.FromToRay[rot2[sq]][rot2[sq2]];
	    bitboards.Bishop315Atak[rot3[sq]][map] =
		bitboards.FromToRay[rot3[sq]][rot3[sq1]] |
		bitboards.FromToRay[rot3[sq]][rot3[sq2]];
      }
   } 

   for (map = 0; map < 256; map++)
   {
      for (sq = A2; sq <= H8; sq++)
      {
	bitboards.Rook00Atak[sq][map] = bitboards.Rook00Atak[sq-8][map] >> 8;
      }

      for (sq1 = B_FILE; sq1 <= H_FILE; sq1++)
      {
	 for (sq2 = 0; sq2 < 64; sq2+=8)
	 {
	    sq = sq2 + sq1;
	    bitboards.Rook90Atak[sq][map] = bitboards.Rook90Atak[sq-1][map] >> 1;
	 }
      }

      for (sq1 = B1, sq2 = H7; sq1 <= H1; sq1++, sq2-=8)
      {
         for (sq = sq1; sq <= sq2; sq += 9)
	 {
	   bitboards.Bishop45Atak[sq][map] = bitboards.Bishop45Atak[sq+8][map] << 8;
	 }
      }
      for (sq1 = A2, sq2 = G8; sq1 <= A8; sq1+=8, sq2--)
      {
         for (sq = sq1; sq <= sq2; sq += 9)
	 {
	    bitboards.Bishop45Atak[sq][map] = 
		(bitboards.Bishop45Atak[sq+1][map] & bitboards.NotBitPosArray[sq1-8]) << 1;
	 }
      }

      for (sq1 = H2, sq2 = B8; sq1 <= H8; sq1+=8, sq2++)
      {
         for (sq = sq1; sq <= sq2; sq += 7)
	 {
	    bitboards.Bishop315Atak[sq][map] = bitboards.Bishop315Atak[sq-8][map] >> 8;
	 }
      }
      for (sq1 = G1, sq2 = A7; sq1 >= A1; sq1--, sq2-=8)
      {
         for (sq = sq1; sq <= sq2; sq += 7)
	 {
	    bitboards.Bishop315Atak[sq][map] = 
		(bitboards.Bishop315Atak[sq+1][map] & bitboards.NotBitPosArray[sq2+8]) << 1;
	 }
      }
   }
}

void InitDistance (void)
/**************************************************************************
 *
 *  There are two arrays dealing with distances.  The distance[s1][s2]
 *  array gives the minimum number of moves a king takes to get from s1
 *  to s2.  The taxicab[s1][s2] array is the taxicab distance.  Example:
 *  distance[A1][B3] = 2;  taxicab[A1][B3] = 3;
 *
 *************************************************************************/
{
   int f, t, j;
   int d1, d2;

   for (f = 0; f < 64; f++)
     for (t = 0; t < 8; t++)
       bitboards.DistMap[f][t] = ULL(0);


   for (f = 0; f < 64; f++)
      for (t = f; t < 64; t++)
      {
         d1 = (t & 0x07) - (f & 0x07);
         if (d1 < 0) d1 = -d1;
         d2 = (t >> 3) - (f >> 3);
         if (d2 < 0) d2 = -d2;
         distance[f][t] = MAX (d1, d2);
         distance[t][f] = MAX (d1, d2);
         taxicab[f][t] = d1 + d2;
         taxicab[t][f] = d1 + d2;
      }

   for (f = 0; f < 64; f++)
     for (t = 0; t < 64; t++)
	 bitboards.DistMap[f][distance[t][f]] |= bitboards.BitPosArray[t];

   for (f = 0; f < 64; f++)
     for (t = 0; t < 8; t++)
       for (j = 0; j < t; j++)
	 bitboards.DistMap[f][t] |= bitboards.DistMap[f][j];
}



void BitBoardInit(){
   InitBitPosArray ();
   InitMoveArray ();
   InitRay ();
   InitFromToRay (); 
   InitRankFileBit ();
   InitPassedPawnMask ();
   InitIsolaniMask ();
   InitSquarePawnMask ();
   InitBitCount ();
   InitRotAtak ();
   InitRandomMasks ();
   InitDistance ();
}
