// includes

#include <pthread.h>
#include <stdarg.h>
#include "common.h"
#include "log.h"

// variables

log_t LogDefault[1];

// functions

void Log(char *fmt, ...){
   va_list ap;
   char buf[MAXSTR];
   va_start(ap, fmt);
   vsnprintf(buf, MAXSTR-1, fmt, ap);
   va_end(ap);
   LogLock(LogDefault);
   LogString(LogDefault, buf);
   LogUnlock(LogDefault);
}

#ifndef PTHREAD_MUTEX_RECURSIVE
#define PTHREAD_MUTEX_RECURSIVE PTHREAD_MUTEX_RECURSIVE_NP
#endif

void LogInit(log_t *logger){
    pthread_mutexattr_t attr;
    logger->filename=NULL;
    logger->ofp=NULL;
    logger->state=false;
    logger->timestamp=false;
    logger->was_newline=true;
    logger->newline_prefix=NULL;
    my_string_set(&logger->newline_prefix,"");
    pthread_mutexattr_init(&attr);
    pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
    pthread_mutex_init(&logger->mutex, &attr);
    pthread_mutexattr_destroy(&attr);
}

void LogLock(log_t *logger){
    pthread_mutex_lock (&logger->mutex);
}

void LogUnlock(log_t *logger){
    pthread_mutex_unlock (&logger->mutex);
}

void LogClear(log_t *logger){
    if(logger->ofp){
	fclose(logger->ofp);
    }
    pthread_mutex_destroy(&logger->mutex);
    LogInit(logger);
}

void LogSetFile(log_t *logger, const char *filename){
    if(logger->ofp){
	fclose(logger->ofp);
	logger->ofp=NULL;
    }
    my_string_set(&logger->filename,filename);
}

void LogSetNewlinePrefix(log_t *logger, const char *s){
    my_string_set(&logger->newline_prefix,s);
}

void LogEnsureNewline(log_t *logger){
    if(!logger->was_newline){
	logger->was_newline=true;
    }
}

void LogSetState(log_t *logger, int state){
    logger->state=state;
    if(logger->ofp){
	fclose(logger->ofp);
	logger->ofp=NULL;
    }
}

void LogSetTimeStamp(log_t *logger, int timestamp){
    logger->timestamp=timestamp;
}

void LogString(log_t *logger, char  *s){
    struct timeval tv;
    char buf1[MAXSTR];
    int i=0;
    char c;
    
    if(!logger->state){ 
	return;
    }
    
    if(!logger->filename){
	return;
    }
    
    if(!logger->ofp){
	logger->ofp=fopen(logger->filename,"a");
	if(!logger->ofp){
	    return;
	}
    }
    
    while((c=s[i++])!='\0'){
	if(logger->was_newline){
	    if(logger->timestamp){
		gettimeofday(&tv, NULL);
		sprintf(buf1, "%010ld.%06ld: ", tv.tv_sec, tv.tv_usec);
		fprintf(logger->ofp, "%s",buf1);
		fflush(logger->ofp);
		logger->was_newline=false;
	    }
	    fprintf(logger->ofp, "%s",logger->newline_prefix);
	}
	fprintf(logger->ofp,"%c",c);
	if(c=='\n'){
	    logger->was_newline=true;
	}
    }
    
}     

