// log.h

#ifndef LOG_H
#define LOG_H

// includes

#include <pthread.h>

// types

typedef struct {
    const char *filename;
    FILE *ofp;
    int state;  // logging on/off
    int timestamp;
    int was_newline;
    const char *newline_prefix;
    pthread_mutex_t mutex;
} log_t;

// variables

extern log_t LogDefault[1];



// functions


void Log(char *fmt, ...);
void LogInit(log_t *logger);
void LogClear(log_t *logger);
void LogSetFile(log_t *logger, const char *filename);
void LogSetNewlinePrefix(log_t *logger, const char *s);
void LogSetState(log_t *logger, int state);
void LogSetTimeStamp(log_t *logger, int timestamp);
void LogString(log_t *logger, char *s);
void LogEnsureNewline(log_t *logger);

/* 
 *  Log() has internal locking. If you do something else 
 *  you should Lock/Unlock.
 */ 

void LogLock(log_t *logger);
void LogUnlock(log_t *logger);

#endif // LOG_H
