/* GNU Chess 5.0 - main.c - entry point
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   GNU Chess is based on the two research programs 
   Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.

   GNU Chess is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GNU Chess is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info: 
   bug-gnu-chess@gnu.org
   cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
*/

#include <time.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <errno.h>                                                              
#include <unistd.h>

#include "common.h"
#include "eval.h"
#include "book.h"
#include "movelist.h"
#include "option.h"
#include "log.h"
#include "ttable.h"

short distance[64][64];
short taxicab[64][64];
unsigned char lzArray[65536];
short directions[64][64];
unsigned char BitCount[65536];


board_t Board[1];
//game_t Game[1];
protocol_t Protocol[1];
HashType hashcode[2][7][64];
HashType ephash[64];
HashType WKCastlehash;
HashType WQCastlehash;
HashType BKCastlehash;
HashType BQCastlehash;
HashType Sidehash;
PawnSlot *PawnTab[2];

// XXX Remaining troublesome globals

volatile unsigned int flags;
unsigned int preanalyze_flags;
unsigned long PHashMask;
unsigned long TotalPawnHashCnt;
unsigned long GoodPawnHashCnt;
BitBoard Ataks[2][7];
BitBoard pinned;
BitBoard pieces[2];



// searchparams_t SearchParams[1];



char *progname;


int slider[8] = { 0, 0, 0, 1, 1, 1, 0, 0 };
int Value[7] = { 0, ValueP, ValueN, ValueB, ValueR, ValueQ, ValueK};
int range[8] = { 0, 0, 0, 1, 1, 1, 0, 0 };
int ptype[2] = { pawn, bpawn };
char algbr[64][3] =
    { "a1", "b1", "c1", "d1", "e1", "f1", "g1", "h1",
      "a2", "b2", "c2", "d2", "e2", "f2", "g2", "h2",
      "a3", "b3", "c3", "d3", "e3", "f3", "g3", "h3",
      "a4", "b4", "c4", "d4", "e4", "f4", "g4", "h4",
      "a5", "b5", "c5", "d5", "e5", "f5", "g5", "h5",
      "a6", "b6", "c6", "d6", "e6", "f6", "g6", "h6",
      "a7", "b7", "c7", "d7", "e7", "f7", "g7", "h7",
      "a8", "b8", "c8", "d8", "e8", "f8", "g8", "h8" 
    };
char algbrfile[9] = "abcdefgh";
char algbrrank[9] = "12345678";
  
char notation[8] = { " PNBRQK" };
char lnotation[8] = { " pnbrqk" };

short Shift00[64] =
    { 56, 56, 56, 56, 56, 56, 56, 56,
      48, 48, 48, 48, 48, 48, 48, 48,
      40, 40, 40, 40, 40, 40, 40, 40,
      32, 32, 32, 32, 32, 32, 32, 32,
      24, 24, 24, 24, 24, 24, 24, 24,
      16, 16, 16, 16, 16, 16, 16, 16,
      8,  8,  8,  8,  8,  8,  8,  8,
      0,  0,  0,  0,  0,  0,  0,  0
    };

int r90[64] =
    { A8, A7, A6, A5, A4, A3, A2, A1,
      B8, B7, B6, B5, B4, B3, B2, B1,
      C8, C7, C6, C5, C4, C3, C2, C1,
      D8, D7, D6, D5, D4, D3, D2, D1,
      E8, E7, E6, E5, E4, E3, E2, E1,
      F8, F7, F6, F5, F4, F3, F2, F1,
      G8, G7, G6, G5, G4, G3, G2, G1,
      H8, H7, H6, H5, H4, H3, H2, H1 };

short Shift90[64] =
    { 0, 8, 16, 24, 32, 40, 48, 56,
      0, 8, 16, 24, 32, 40, 48, 56,
      0, 8, 16, 24, 32, 40, 48, 56,
      0, 8, 16, 24, 32, 40, 48, 56,
      0, 8, 16, 24, 32, 40, 48, 56,
      0, 8, 16, 24, 32, 40, 48, 56,
      0, 8, 16, 24, 32, 40, 48, 56,
      0, 8, 16, 24, 32, 40, 48, 56
    };

int r45[64] =
    { E4, F3, H2, C2, G1, D1, B1, A1,
      E5, F4, G3, A3, D2, H1, E1, C1,
      D6, F5, G4, H3, B3, E2, A2, F1, 
      B7, E6, G5, H4, A4, C3, F2, B2,
      G7, C7, F6, H5, A5, B4, D3, G2, 
      C8, H7, D7, G6, A6, B5, C4, E3, 
      F8, D8, A8, E7, H6, B6, C5, D4, 
      H8, G8, E8, B8, F7, A7, C6, D5 };

short Shift45[64] =
    { 28, 36, 43, 49, 54, 58, 61, 63,
      21, 28, 36, 43, 49, 54, 58, 61,
      15, 21, 28, 36, 43, 49, 54, 58,
      10, 15, 21, 28, 36, 43, 49, 54,
      6, 10, 15, 21, 28, 36, 43, 49,
      3,  6, 10, 15, 21, 28, 36, 43,
      1,  3,  6, 10, 15, 21, 28, 36,
      0,  1,  3,  6, 10, 15, 21, 28 };

int Mask45[64] =
    { 0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07, 0x03, 0x01,
      0x7F, 0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07, 0x03, 
      0x3F, 0x7F, 0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07, 
      0x1F, 0x3F, 0x7F, 0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 
      0x0F, 0x1F, 0x3F, 0x7F, 0xFF, 0x7F, 0x3F, 0x1F, 
      0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF, 0x7F, 0x3F, 
      0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF, 0x7F, 
      0x01, 0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF };

int r315[64] =
    { A1, C1, F1, B2, G2, E3, D4, D5,
      B1, E1, A2, F2, D3, C4, C5, C6,
      D1, H1, E2, C3, B4, B5, B6, A7,
      G1, D2, B3, A4, A5, A6, H6, F7,
      C2, A3, H3, H4, H5, G6, E7, B8,
      H2, G3, G4, G5, F6, D7, A8, E8,
      F3, F4, F5, E6, C7, H7, D8, G8,
      E4, E5, D6, B7, G7, C8, F8, H8 };

short Shift315[64] =
    { 63, 61, 58, 54, 49, 43, 36, 28,
      61, 58, 54, 49, 43, 36, 28, 21,
      58, 54, 49, 43, 36, 28, 21, 15,
      54, 49, 43, 36, 28, 21, 15, 10,
      49, 43, 36, 28, 21, 15, 10,  6,
      43, 36, 28, 21, 15, 10,  6,  3,
      36, 28, 21, 15, 10,  6,  3,  1,
      28, 21, 15, 10,  6,  3,  1,  0 };

int Mask315[64] =
    { 0x01, 0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF,
      0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF, 0x7F,
      0x07, 0x0F, 0x1F, 0x3F, 0x7F, 0xFF, 0x7F, 0x3F,
      0x0F, 0x1F, 0x3F, 0x7F, 0xFF, 0x7F, 0x3F, 0x1F,
      0x1F, 0x3F, 0x7F, 0xFF, 0x7F, 0x3F, 0x1F, 0x0F,
      0x3F, 0x7F, 0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07,
      0x7F, 0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07, 0x03,
      0xFF, 0x7F, 0x3F, 0x1F, 0x0F, 0x07, 0x03, 0x01 };

int rank6[2] = { 5, 2 };
int rank7[2] = { 6, 1 };
int rank8[2] = { 7, 0 };


void gnuchess_set_option(const char *name, const char *value){ 
    Log("Setting %s to %s\n",name,value);
    OptionSet(Option,name,value);
    if(my_string_case_equal(name,"BookFile")){
	BookSetFile(Book,OptionGetString(Option,"BookFile"));
    }else if(my_string_case_equal(name,"Log")){
	LogSetState(LogDefault,OptionGetBool(Option,"Log"));
    }else if(my_string_case_equal(name,"LogFile")){
	LogSetFile(LogDefault,OptionGetString(Option,"LogFile"));
    }else if(my_string_case_equal(name,"ClearHash")){
	Log("Clearing transposition table\n");
	TTableZero(TTable);
	PTClear();
    }
    SearchInit(Option);

}

void DoComputerMove(){      

    board_t board[1];
    movelist_t movelist[1];
    searchstats_t searchstats[1];
    searchparams_t searchparams[1];

    int state;
    char SANmv[SANSZ];
    char LANmv[SANSZ];
    leaf *l;

    SearchStatsInit(searchstats);
    SearchParamsCreate(searchparams,Protocol,Option);
  
    OutputConsole("Thinking...\n");
  
    BoardFromGame(board,Protocol->game,BOARD_LAST);
    ASSERT(BoardValidate(board)==0);
  
  
    ASSERT(Protocol->computer==board->side);
    l=NULL;
    GenMoves (board,movelist);
    if (movelist->gencnt == 1){
	OutputConsole("Single move!\n");
	l= movelist->moves;
	searchstats->RootPV=l->move;
	searchstats->ElapsedTime=0.0;
    }
    if(!l && OptionGetBool(Option,"Book")){
	BookQuery(Book,board,movelist);
	if(OptionGetBool(Option,"BookRandom")){
	    l=MoveListPickRandom(movelist);
	}else{
	    l=MoveListPickBest(movelist);
	}
	if(l){
	    OutputConsole("BookMove!\n");
	    if(Protocol->console){
		MoveListShow(board,movelist);
	    }
	    searchstats->RootPV=l->move;
	    searchstats->ElapsedTime=0.0;
	}
    }
    if(!l){
	Iterate (board, searchparams, searchstats);
    }
    MoveToSAN (board,searchstats->RootPV,SANmv);
    Protocol->game->boards[board->GameCnt-Protocol->game->InitialGameCnt].move=searchstats->RootPV;
    Protocol->game->boards[board->GameCnt-Protocol->game->InitialGameCnt].et = searchstats->ElapsedTime;
    MoveMake(board, searchstats->RootPV);
    //    Protocol->game->boards[Protocol->game->RealGameCnt-Protocol->game->InitialGameCnt].move=searchstats->RootPV;
    Protocol->game->RealGameCnt = board->GameCnt;
    if (Protocol->TCMove) {
	if(Protocol->game->RealGameCnt>=Protocol->NextPeriod){
	    Protocol->NextPeriod+=2*Protocol->TCMove;
	}
    }
    Protocol->TimeLimit[board->side] -= searchstats->ElapsedTime;
    if (Protocol->TCinc != 0){
	Protocol->TimeLimit[board->side] += Protocol->TCinc;
    }
    MoveToLAN(searchstats->RootPV,LANmv);
    Output("move %s\n", LANmv);
    if (Protocol->console){
	BoardShow (board);
    }
    OutputConsole("\nMy move is : %s\n", SANmv);
  
  
    state=BoardState(board);
    if(state==BOARD_WHITE_WINS){
	Output("1-0 {White Mates}\n");
	Protocol->game->result=R_WHITE_WINS;
    }else if(state==BOARD_BLACK_WINS){
	Output("0-1 {Black Mates}\n");
	Protocol->game->result=R_BLACK_WINS;
    }else if(state==BOARD_STALEMATE){
	Output("1/2-1/2 {stalemate}\n");
	Protocol->game->result=R_DRAW;
    }else if(state==BOARD_50MOVERULE){
	Output("1/2-1/2 {Draw by 50 move rule}\n");
	Protocol->game->result=R_DRAW;
    }else if(state==BOARD_3FOLD_REPETITION){
	Output("1/2-1/2 {Draw by 3-fold repetition}\n");
	Protocol->game->result=R_DRAW;
    }
  
    CLEAR (flags, THINK);
  
}

int main (int argc, char *argv[])
{
    int i;
    char SANmv[SANSZ];
    char LANmv[SANSZ];

    /*
     * Parse command line arguments conforming with getopt_long syntax
     * Note: we have to support "xboard" and "post" as bare strings
     * for backward compatibility.
     */
 
    int c;
    int opt_help = 0, opt_version = 0, opt_post = 0, opt_xboard = 0, opt_hash = 0, opt_easy = 0, opt_manual = 0;
    int state;
    char *endptr;
    int do_ponder;
    searchstats_t searchstats[1];
    searchdata_t searchdata[1];
    board_t board[1];
    movelist_t movelist[1];
    leaf *l;
  
    Initialize ();

    progname = argv[0]; /* Save in global for cmd_usage */

    while (1)
	{
	    static struct option long_options[] =
		{
		    {"hashsize", 1, 0, 's'},
		    {"version", 0, 0, 'v'},
		    {"help", 0, 0, 'h'},
		    {"xboard", 0, 0, 'x'},
		    {"post", 0, 0, 'p'},
		    {"easy", 0, 0, 'e'},
		    {"manual", 0, 0, 'm'},
		    {0, 0, 0, 0}
		};
 
	    /* getopt_long stores the option index here. */ 

	    int option_index = 0;
 
	    c = getopt_long (argc, argv, "ehmpvxs:D:",
			     long_options, &option_index);
 
	    /* Detect the end of the options. */
	    if (c == -1)
		break;

	    /* 
	     * Options with a straight flag, could use getoopt_long
	     * flag setting but this is more "obvious" and easier to
	     * modify.
	     */
	    switch (c)
		{
		case 'v':
		    opt_version = 1;
		    break;
		case 'h':
		    opt_help = 1;
		    break;
		case 'x':
		    opt_xboard = 1;
		    break;
		case 'p':
		    opt_post = 1;
		    break;
		case 'e':
		    opt_easy = 1;
		    break;
		case 'm':
		    opt_manual = 1;
		    break;
		case 's':    
		    if  ( optarg == NULL ){ /* we have error such as two -s */
			opt_help = 1;
			break;
		    }
		    errno = 0; /* zero error indicator */
		    opt_hash = strtol (optarg, &endptr, 10);
		    if ( errno != 0 || *endptr != '\0' ){
			OutputConsole("Hashsize out of Range or Invalid\n");
			return(1);
		    }
		    break;
		case '?': /* On error give help - getopt does a basic message. */
		    opt_help = 1;
		    break;
		default:
		    puts ("Option Processing Failed\n");
		    abort();
		}
	} /* end of getopt_long style parsing */

  
      /* Handle old style command line options */
    if (argc > 1) {
	for (i = 0; i < argc; i++) {
	    if (strcmp(argv[i],"xboard") == 0) {
		opt_xboard=1;
	    } else if (strcmp(argv[i],"post") == 0) {
		opt_post=1;
	    }
	}
    }

    if (opt_xboard == 1){
	Protocol->console=0;
    }
    if (opt_post == 1){
	SET (flags, POST);	
    }
    if (opt_manual ==1){
	SET (flags, MANUAL);
    }
    if(Protocol->console){
	cmd_version();
    }
  
    /* If the version option was specified we can exit here */
    if (opt_version == 1)
	return(0);
  
    /* If a usage statement is required output it here */
    if (opt_help == 1){
	cmd_usage();
	return (1); /* Maybe an error if due to bad arguments. */
    }

    // move this init stuff to "initialize!
  
    // [VdB] New option code. Here and elsewhere.


    if ( opt_hash != 0){
	TTableSetSize(TTable,opt_hash);
    }

    if ( opt_easy == 0)
	SET (flags, HARD);


    // XXX Replace by something like ShowState
    
    if (Protocol->console) {
      BoardFromGame(board,Protocol->game,BOARD_LAST);
      BoardShow (board);
    }

    while (!(flags & QUIT)) {
	Log("Waiting for input...\n");
	/* prompt */
	wait_for_input();
	Log("Parsing input...\n");
	parse_input();
	Log("input_status = %d\n", input_status);
	Log("Waking up input...\n");
	input_wakeup();
	Log("input_status = %d\n", input_status);
	Log("Parsing input...\n");
	if ((flags & THINK) && 
	    !(flags & MANUAL) && 
	    !(Protocol->game->result!=R_NORESULT) && 
	    !(flags & QUIT)) {
	    DoComputerMove();
	}

	do_ponder=false; // make the compiler happy.

	if(input_status!=INPUT_NONE){
	    do_ponder=false;
	}else if(flags & QUIT){
	    do_ponder=false;
	}else if(flags & ANALYZE){
	    do_ponder=true;
	}else if(Protocol->game->RealGameCnt==Protocol->game->InitialGameCnt){
	    do_ponder=false;
	}else if(flags & MANUAL){
	    do_ponder=false;
	}else if(Protocol->game->result!=R_NORESULT){
	    do_ponder=false;
	}else if(flags & HARD){
	    do_ponder=true;
	}
	if(do_ponder){
	    searchparams_t searchparams[1];
	    BoardFromGame(board,Protocol->game,BOARD_LAST);
	    ASSERT(BoardValidate(board)==0);
	    SearchParamsCreate(searchparams,Protocol,Option);
	    ponder(board,searchparams, searchstats);
	}
    }

    CleanupInput();

    /*  Some cleaning up  */
    TTableClear(TTable);

    return (0);
}
