/* GNU Chess 5.0 - output.c - output code
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   GNU Chess is based on the two research programs 
   Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.

   GNU Chess is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GNU Chess is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info: 
     bug-gnu-chess@gnu.org
     cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
*/
/*
 *
 */

#include <stdio.h>
#include "common.h"
#include "bitboard.h"

void ShowTime (void)
/**************************************************************************
 *
 *  Print out the time settings.
 *
 **************************************************************************/
{
}



void ShowSmallBoard (board_t *board)     
/*****************************************************************************
 *
 *  Display the board.  Not only that but display some useful information
 *  like whether enpassant is legal and castling state.
 *
 *****************************************************************************/ 
{
   int r, c, sq;

   Output("\n");
   if (board->side == white)
      Output("white  ");
   else
      Output("black  ");

   if (board->flag & WKINGCASTLE)
      Output("K");
   if (board->flag & WQUEENCASTLE)
      Output("Q");
   if (board->flag & BKINGCASTLE)
      Output("k");
   if (board->flag & BQUEENCASTLE)
      Output("q");

   if (board->ep > -1)
      Output("  %s", algbr[board->ep]);

   Output("\n");
   for (r = 56; r >= 0; r -= 8)
   {
      for (c = 0; c < 8; c++)
      {
         sq = r + c;
         if (board->b[white][pawn] & bitboards.BitPosArray[sq])
            Output("P ");
         else if (board->b[white][knight] & bitboards.BitPosArray[sq])
            Output("N ");
         else if (board->b[white][bishop] & bitboards.BitPosArray[sq])
            Output("B ");
         else if (board->b[white][rook]   & bitboards.BitPosArray[sq])
            Output("R ");
         else if (board->b[white][queen]  & bitboards.BitPosArray[sq])
            Output("Q ");
         else if (board->b[white][king]   & bitboards.BitPosArray[sq])
            Output("K ");
         else if (board->b[black][pawn]   & bitboards.BitPosArray[sq])
            Output("p ");
         else if (board->b[black][knight] & bitboards.BitPosArray[sq])
            Output("n ");
         else if (board->b[black][bishop] & bitboards.BitPosArray[sq])
            Output("b ");
         else if (board->b[black][rook]   & bitboards.BitPosArray[sq])
            Output("r ");
         else if (board->b[black][queen]  & bitboards.BitPosArray[sq])
            Output("q ");
         else if (board->b[black][king]   & bitboards.BitPosArray[sq])
            Output("k ");
         else
            Output(". ");
      }
      Output("\n");
   }
   Output("\n");
}
  


void ShowBitBoard (BitBoard *b)
/*****************************************************************************
 *
 * Just to print a lousy ascii board  
 *
 *****************************************************************************/
{
   int r, c;

   Output("\n");
   for (r = 56; r >= 0; r -= 8)
   {
      for (c = 0; c < 8; c++)
      {
         if (*b & bitboards.BitPosArray[r + c])
            Output("1 ");
         else
            Output(". "); 
      }
      Output("\n");
   }
   Output("\n");
}



void ShowCBoard (board_t *board)
/*****************************************************************************
 *
 *
 *
 *****************************************************************************/
{
   int r, c;

   for (r = 56; r >= 0; r -= 8)
   {
      for (c = 0; c < 8; c++)
      {
         Output("%2c ", board->cboard[r + c] ? notation[board->cboard[r+c]] : '.');
      }
      Output("\n");
   }
   Output("\n");
}


void ShowMvboard (board_t *board)
/*****************************************************************************
 *
 *  Print the Mvboard[] array.
 *
 *****************************************************************************/
{
   int r, c;

   for (r = 56; r >= 0; r -= 8)
   {
      for (c = 0; c < 8; c++)
      {
         Output("%2d ", board->Mvboard[r + c]);
      }
      Output("\n");
   }
   Output("\n");
}
   

