/* GNU Chess 5.0 - quiesce.c - quiescence search code
   Copyright (c) 1999-2002 Free Software Foundation, Inc.

   GNU Chess is based on the two research programs 
   Cobalt by Chua Kong-Sian and Gazebo by Stuart Cracraft.

   GNU Chess is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GNU Chess is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GNU Chess; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Contact Info: 
   bug-gnu-chess@gnu.org
   cracraft@ai.mit.edu, cracraft@stanfordalumni.org, cracraft@earthlink.net
*/
/*
 *
 */

#include <stdio.h>
#include "common.h"
#include "move.h"
#include "ttable.h"

int Quiesce (board_t *board, 
	     searchparams_t *searchparams,
	     searchdata_t *searchdata, 
	     searchstats_t *searchstats, 
	     uint8_t ply, 
	     int alpha, 
	     int beta)
/**************************************************************************
 *
 *  Our quiescent search.  This quiescent search is able to recognize
 *  mates.
 *
 **************************************************************************/
{
    uint8_t side, xside;
    int best, delta, score, savealpha, pat, swapoff;
    leaf *p, *pbest;
    movelist_t movelist[1];
    undo_t undo[1];
    // [VdB]

    searchstats->QuiesCnt++;

    // emergency exit

    if(ply>=MAXPLYDEPTH){
	return board->in_check?alpha:Evaluate(board, 
					      searchparams,
					      searchdata, 
					      searchstats, 
					      alpha,
					      beta);
    }

    if (EvaluateDraw (board)){
	return (DRAWSCORE);
    }

    side = board->side;
    xside = 1^side;
    best=-INFINITY;
    searchdata->InChk[ply] = board->in_check;
    if (searchdata->InChk[ply])  {
	GenPseudoCheckEscapes (board,movelist);
	if (movelist->gencnt==0){
	    return (-MATE+ply-2);
	}
	SortMoves (board,searchdata,movelist,ply);
    } else  {
	best = Evaluate (board, searchparams, searchdata, searchstats, alpha, beta);
	pat=best;
	if (best >= beta){
	    return (best);
	}
	GenPseudoCaptures (board,movelist);
	if (movelist->gencnt==0){
	    return (best);
	}
	SortCaptures (board,movelist); 
    }

    savealpha = alpha;
    pbest = NULL;
    alpha = MAX(best, alpha);
	 
    for (p = movelist->moves; p < movelist->moves+movelist->gencnt; p++){
	MoveListPickSel(movelist,p);

	if (!searchdata->InChk[ply]){
	    swapoff=SwapOff (board,p->move);
	    // p->score==-INFINITY should mean negative swapoff value: not true it seems
	    // debug this
	    if( swapoff<0 || p->score==-INFINITY || pat+swapoff< alpha-150){
		continue;
	    }
	}


	MoveMakeS (board, &p->move, undo);
	if (board->xin_check){
		MoveUnMakeS (board, undo);
		continue;
	}
	score = -Quiesce (board, 
			  searchparams,
			  searchdata, 
			  searchstats, 
			  ply+1, 
			  -beta, 
			  -alpha);
	MoveUnMakeS (board, undo);
	if (score > best) {
	    best = score;
	    pbest = p;
	    if (best >= beta){
		goto done;
	    }
	    alpha = MAX (alpha, best);
	}
    }

 done:
    // if (flags & USEHASH && pbest != NULL){
    //    TTablePut (TTable,board,  0, ply, savealpha, beta, best, pbest->move);
    //}
    return (best);
}
 
